@@ -15,7 +15,3 @@ hilight("x = 1+1  # a comment")  # default markup
 hilight("x = 1+1  # a comment", markup = m)  # custom markup
 
 
-## ----include=FALSE-------------------------------------------------------
-options(markdown.HTML.header = system.file('misc', 'vignette.css', package='knitr'))
-
-
